const join = require('path').join
const fs = require('fs')
const fm = require('front-matter')
const pathToRegexp = require('path-to-regexp')
const paramCase = require('param-case')

/**
 * Returns 2D array of content data, each with its respective
 * registered directory, options, and nested file data.
 *
 * The data includes both the data that will be used to create the route
 * and the data that will be injected into the component's layout.
 *
 * You also have the option of only requesting the data of a specific
 * registered direction.
 */
module.exports = function getPagesData (config, onlyDir) {
  const srcPath = config.contentPath

  let pagesData = {}
  config.dirs.forEach(type => { // every registered directory
    const dir = join('/', type[0])
    if (!onlyDir || dir === onlyDir) {
      const opts = getDirOpts(type[1] || {}, config)
      const otherDirs = getOtherRegisteredDirs(dir, config.dirs)
      const dirData = getFromDir(srcPath, dir, otherDirs, opts)
      Object.keys(dirData).forEach(key => pagesData[key] = dirData[key])
    }
  })
  return pagesData
}

/**
 * Initiates fecthing of pages from specified directory.
 */
function getFromDir(srcPath, dir, blacklist, opts) {
  let pagesData = {}

  /**
   * Recursively get all content files with their respective metadata.
   */
  function getFilesData (contentDir, filesData = []) {
    const contentPath = join(srcPath, contentDir)
    fs.readdirSync(contentPath).forEach(stat => {
      const statPath = join(contentPath, stat)
      if(fs.statSync(statPath).isDirectory()) {
        const nestedContentDir = join(contentDir, stat)
        if (!(blacklist.indexOf(nestedContentDir) > -1)) {
          getFilesData(nestedContentDir, filesData)
        }
      } else {
        pagesData[statPath] = getFileData(statPath, contentDir, opts)
      }
    })
  }

  const path = join(srcPath, dir)
  fs.readdirSync(path).forEach(stat => {
    const statPath = join(path, stat)
    if(fs.statSync(statPath).isDirectory()) { // Nested Files
      const dirSection = join(dir, stat)

      if (!(blacklist.indexOf(dirSection) > -1)) {
        getFilesData(dirSection)
      }
    } else { // Top Level file
      const statSection = join(dir, stat)
      pagesData[statPath] = getFileData(statPath, dir, opts)
    }
  })
  return pagesData
}


/**
 * Gets data required to create component's route and page layout.
 *
 * Data is retrieved either via 1) front-matter 2) file name 3) config options.
 */
function getFileData(filePath, section, opts) {
  const source = fs.readFileSync(filePath, 'utf-8')
  const metadata = fm(source).attributes

  const fileName = filePath
    .replace(/^.*[\\\/]/, '')  //remove path
    .replace(/(.)[^.]+$/, '')  // remove ext
  const fileDate = fileName.match(/!?(\d{4}-\d{2}-\d{2})/)

  let permalink
  let date
  if (metadata.permalink) {
    permalink = metadata.permalink
  } else {
    slug = fileName.replace(/!?(\d{4}-\d{2}-\d{2}-)/, '') // remove date
    const urlOpts = {
      section,
      slug: paramCase(metadata.slug || slug)
    }

    if (opts.isPost) {
      date = fileDate[0]
      const dateData = date.split('-')
      urlOpts.year = dateData[0]
      urlOpts.month = dateData[1]
      urlOpts.day = dateData[2]
    }

    const toPath = pathToRegexp.compile(opts.permalink)
    permalink = toPath(urlOpts, { pretty: true })
      .replace(/%2F/gi, "/") // make url encoded slash pretty
  }

  const pageData = Object.assign({}, opts.data, metadata, {
    date,
    permalink,
    section
  })

  return {
    fileName,
    section,
    options: opts,
    page: pageData,
    route: {
      path: join("/", permalink),
      component: filePath
    }
  }
}


/**
 * Gets content options via 1) directory options 2) global content config.
 */
function getDirOpts (dir, config) {
  const mergedData = Object.assign({}, config.data, dir.data)
  const opts = Object.assign({}, config, dir, { data: mergedData })

  return {
    route: join('/', opts.route),
    permalink: opts.permalink,
    isPost: !(opts.isPost === false),
    data: opts.data
  }
}

/**
 * Gets all the registered content directory types.
 */
function  getOtherRegisteredDirs (currDir, contentTypes) {
  const dirs = []
  contentTypes.forEach(type => {
    const dir = type[0]
    if (dir !== currDir) dirs.push(join('/' + dir))
  })
  return dirs
}
